@@ -12,11 +12,11 @@
  */
 package org.apache.tuweni.bytes;
 
-import static com.google.common.base.Preconditions.checkArgument;
-import static com.google.common.base.Preconditions.checkElementIndex;
-import static com.google.common.base.Preconditions.checkNotNull;
 import static java.lang.String.format;
 import static java.nio.ByteOrder.BIG_ENDIAN;
+import static org.apache.tuweni.bytes.Checks.checkArgument;
+import static org.apache.tuweni.bytes.Checks.checkElementIndex;
+import static org.apache.tuweni.bytes.Checks.checkNotNull;
 
 import java.io.IOException;
 import java.io.UncheckedIOException;
@@ -133,6 +133,21 @@ public interface Bytes extends Comparable<Bytes> {
     return ConcatenatedBytes.wrap(values);
   }
 
+  /**
+   * Wrap a list of other values into a concatenated view.
+   *
+   * <p>
+   * Note that the values are not copied and thus any future update to the values will be reflected in the returned
+   * value. If copying the inputs is desired, use {@link #concatenate(Bytes...)}.
+   *
+   * @param values The values to wrap.
+   * @return A value representing a view over the concatenation of all {@code values}.
+   * @throws IllegalArgumentException if the result overflows an int.
+   */
+  static Bytes wrap(List<Bytes> values) {
+    return ConcatenatedBytes.wrap(values);
+  }
+
   /**
    * Create a value containing the concatenation of the values provided.
    *
@@ -599,9 +614,23 @@ public interface Bytes extends Comparable<Bytes> {
    * @return a value filled with a fixed byte
    */
   static Bytes repeat(byte b, int size) {
-    byte[] buffer = new byte[size];
-    Arrays.fill(buffer, b);
-    return Bytes.wrap(buffer);
+    return new ConstantBytesValue(b, size);
+  }
+
+  /**
+   * Splits a Bytes object into Bytes32 objects. If the last element is not exactly 32 bytes, it is right padded with
+   * zeros.
+   *
+   * @param bytes the bytes object to segment
+   * @return an array of Bytes32 objects
+   */
+  static Bytes32[] segment(Bytes bytes) {
+    int segments = (int) Math.ceil(bytes.size() / 32.0);
+    Bytes32[] result = new Bytes32[segments];
+    for (int i = 0; i < segments; i++) {
+      result[i] = Bytes32.rightPad(bytes.slice(i * 32, Math.min(32, bytes.size() - i * 32)));
+    }
+    return result;
   }
 
   /**
@@ -1362,9 +1391,7 @@ public interface Bytes extends Comparable<Bytes> {
         destination.size() - destinationOffset,
         destinationOffset);
 
-    for (int i = 0; i < size; i++) {
-      destination.set(destinationOffset + i, get(i));
-    }
+    destination.set(destinationOffset, this);
   }
 
   /**
@@ -1405,18 +1432,37 @@ public interface Bytes extends Comparable<Bytes> {
    */
   default <T extends Appendable> T appendHexTo(T appendable) {
     try {
-      int size = size();
-      for (int i = 0; i < size; i++) {
-        byte b = get(i);
-        appendable.append(AbstractBytes.HEX_CODE[b >> 4 & 15]);
-        appendable.append(AbstractBytes.HEX_CODE[b & 15]);
-      }
+      appendable.append(toFastHex(false));
       return appendable;
     } catch (IOException e) {
       throw new UncheckedIOException(e);
     }
   }
 
+  default String toFastHex(boolean prefix) {
+
+    int offset = prefix ? 2 : 0;
+
+    int resultSize = (size() * 2) + offset;
+
+    char[] result = new char[resultSize];
+
+    if (prefix) {
+      result[0] = '0';
+      result[1] = 'x';
+    }
+
+    for (int i = 0; i < size(); i++) {
+      byte b = get(i);
+      int pos = i * 2;
+      result[pos + offset] = AbstractBytes.HEX_CODE_AS_STRING.charAt(b >> 4 & 15);
+      result[pos + offset + 1] = AbstractBytes.HEX_CODE_AS_STRING.charAt(b & 15);
+    }
+
+    return new String(result);
+
+  }
+
   /**
    * Return the number of bytes in common between this set of bytes and another.
    *
@@ -1459,6 +1505,21 @@ public interface Bytes extends Comparable<Bytes> {
     return Bytes.EMPTY;
   }
 
+  /**
+   * Return a slice of representing the same value but without any trailing zero bytes.
+   *
+   * @return {@code value} if its right-most byte is non zero, or a slice that exclude any trailing zero bytes.
+   */
+  default Bytes trimTrailingZeros() {
+    int size = size();
+    for (int i = size - 1; i >= 0; i--) {
+      if (get(i) != 0) {
+        return slice(0, i + 1);
+      }
+    }
+    return Bytes.EMPTY;
+  }
+
   /**
    * Update the provided message digest with the bytes of this value.
    *
@@ -1541,7 +1602,7 @@ public interface Bytes extends Comparable<Bytes> {
    * @return This value represented as hexadecimal, starting with "0x".
    */
   default String toHexString() {
-    return appendHexTo(new StringBuilder("0x")).toString();
+    return toFastHex(true);
   }
 
   /**
@@ -1550,7 +1611,7 @@ public interface Bytes extends Comparable<Bytes> {
    * @return This value represented as hexadecimal, with no prefix.
    */
   default String toUnprefixedHexString() {
-    return appendHexTo(new StringBuilder()).toString();
+    return toFastHex(false);
   }
 
   default String toEllipsisHexString() {
@@ -1558,19 +1619,24 @@ public interface Bytes extends Comparable<Bytes> {
     if (size < 6) {
       return toHexString();
     }
-    StringBuilder appendable = new StringBuilder("0x");
+    char[] result = new char[12];
+    result[0] = '0';
+    result[1] = 'x';
     for (int i = 0; i < 2; i++) {
       byte b = get(i);
-      appendable.append(AbstractBytes.HEX_CODE[b >> 4 & 15]);
-      appendable.append(AbstractBytes.HEX_CODE[b & 15]);
+      int pos = (i * 2) + 2;
+      result[pos] = AbstractBytes.HEX_CODE_AS_STRING.charAt(b >> 4 & 15);
+      result[pos + 1] = AbstractBytes.HEX_CODE_AS_STRING.charAt(b & 15);
     }
-    appendable.append("..");
+    result[6] = '.';
+    result[7] = '.';
     for (int i = 0; i < 2; i++) {
       byte b = get(i + size - 2);
-      appendable.append(AbstractBytes.HEX_CODE[b >> 4 & 15]);
-      appendable.append(AbstractBytes.HEX_CODE[b & 15]);
+      int pos = (i * 2) + 8;
+      result[pos] = AbstractBytes.HEX_CODE_AS_STRING.charAt(b >> 4 & 15);
+      result[pos + 1] = AbstractBytes.HEX_CODE_AS_STRING.charAt(b & 15);
     }
-    return appendable.toString();
+    return new String(result);
   }
 
   /**
@@ -1579,7 +1645,7 @@ public interface Bytes extends Comparable<Bytes> {
    * @return This value represented as a minimal hexadecimal string (without any leading zero).
    */
   default String toShortHexString() {
-    StringBuilder hex = appendHexTo(new StringBuilder());
+    String hex = toFastHex(false);
 
     int i = 0;
     while (i < hex.length() && hex.charAt(i) == '0') {
@@ -1599,13 +1665,13 @@ public interface Bytes extends Comparable<Bytes> {
     if (Bytes.EMPTY.equals(this)) {
       return "0x0";
     }
-    StringBuilder hex = appendHexTo(new StringBuilder());
+    String hex = toFastHex(false);
 
     int i = 0;
     while (i < hex.length() - 1 && hex.charAt(i) == '0') {
       i++;
     }
-    return "0x" + hex.substring(hex.charAt(hex.length() - 1) == '0' ? i : i++);
+    return "0x" + hex.substring(i);
   }
 
   /**
