@@ -17,16 +17,17 @@
 package org.apache.tuweni.ethclientui
 
 import org.apache.tuweni.ethclient.EthereumClient
-import javax.servlet.ServletContext
-import javax.ws.rs.GET
-import javax.ws.rs.Path
-import javax.ws.rs.Produces
-import javax.ws.rs.core.MediaType
+import jakarta.servlet.ServletContext
+import jakarta.ws.rs.GET
+import jakarta.ws.rs.Path
+import jakarta.ws.rs.Produces
+import jakarta.ws.rs.core.Context
+import jakarta.ws.rs.core.MediaType
 
 @Path("config")
 class ConfigurationService {
 
-  @javax.ws.rs.core.Context
+  @Context
   var context: ServletContext? = null
 
   @GET
