@@ -127,7 +127,8 @@ class EthereumClientConfig(private var config: Configuration = Configuration.emp
         section,
         sectionConfig.getString("peerRepository"),
         sectionConfig.getString("enrLink"),
-        sectionConfig.getLong("pollingPeriod")
+        sectionConfig.getLong("pollingPeriod"),
+        if (sectionConfig.contains("dnsServer")) sectionConfig.getString("dnsServer") else null,
       )
     }
   }
@@ -193,11 +194,32 @@ class EthereumClientConfig(private var config: Configuration = Configuration.emp
       return emptyList()
     }
     return synchronizers.map { section ->
-      val sectionConfig = config.getConfigurationSection("proxy.$section")
+      val sectionConfig = config.getConfigurationSection("synchronizer.$section")
       SynchronizerConfigurationImpl(
-        section, SynchronizerType.valueOf(sectionConfig.getString("type")),
+        section,
+        SynchronizerType.valueOf(sectionConfig.getString("type")),
+        sectionConfig.getString("repository"),
+        sectionConfig.getString("peerRepository"),
+        sectionConfig.getString("rlpxService"),
         UInt256.valueOf(sectionConfig.getLong("from")),
-        UInt256.valueOf(sectionConfig.getLong("to"))
+        UInt256.valueOf(sectionConfig.getLong("to")),
+        sectionConfig.getString("fromRepository"),
+      )
+    }
+  }
+
+  fun validators(): List<ValidatorConfiguration> {
+    val validators = config.sections("validator")
+    if (validators == null || validators.isEmpty()) {
+      return emptyList()
+    }
+    return validators.map { section ->
+      val sectionConfig = config.getConfigurationSection("validator.$section")
+      ValidatorConfigurationImpl(
+        section, ValidatorType.valueOf(sectionConfig.getString("type")),
+        UInt256.valueOf(sectionConfig.getLong("from")),
+        UInt256.valueOf(sectionConfig.getLong("to")),
+        sectionConfig.getInteger("chainId"),
       )
     }
   }
@@ -249,6 +271,7 @@ class EthereumClientConfig(private var config: Configuration = Configuration.emp
       dnsSection.addString("enrLink", null, "DNS domain to query for records", null)
       dnsSection.addLong("pollingPeriod", 50000, "Polling period to refresh DNS records", null)
       dnsSection.addString("peerRepository", "default", "Peer repository to which records should go", null)
+      dnsSection.addString("dnsServer", null, "DNS Server address to use, will use system default if null", null)
 
       val staticPeers = SchemaBuilder.create()
       staticPeers.addListOfString(
@@ -322,11 +345,23 @@ class EthereumClientConfig(private var config: Configuration = Configuration.emp
         "type",
         "status",
         "Synchronizer type",
-        PropertyValidator.anyOf("status", "parent", "best")
+        PropertyValidator.anyOf("status", "parent", "best", "canonical")
       )
       synchronizersSection.addLong("from", 0L, "Start block to sync from", PropertyValidator.isGreaterOrEqual(0L))
       synchronizersSection.addLong("to", 0L, "End block to sync to", PropertyValidator.isGreaterOrEqual(0L))
+      synchronizersSection.addString("repository", "default", "Blockchain repository to use", null)
+      synchronizersSection.addString("rlpxService", "default", "RLPx service to use for requests with this synchronizer", null)
+      synchronizersSection.addString("peerRepository", "default", "Peer repository to use for requests with this synchronizer", null)
+      synchronizersSection.addString("fromRepository", null, "(only for canonical) Repository to sync from", null)
 
+      val validatorsSection = SchemaBuilder.create()
+      validatorsSection.addString(
+        "type",
+        "evm",
+        "Validator type",
+        PropertyValidator.anyOf("header", "difficulty", "evm")
+      )
+      validatorsSection.addInteger("chainId", 1, "chain id to validate with", null)
       val builder = SchemaBuilder.create()
       builder.addSection("metrics", metricsSection.toSchema())
       builder.addSection("storage", storageSection.toSchema())
@@ -338,6 +373,7 @@ class EthereumClientConfig(private var config: Configuration = Configuration.emp
       builder.addSection("proxy", proxiesSection.toSchema())
       builder.addSection("peerRepository", peerRepositoriesSection.toSchema())
       builder.addSection("synchronizer", synchronizersSection.toSchema())
+      builder.addSection("validators", validatorsSection.toSchema())
 
       return builder.toSchema()
     }
@@ -392,6 +428,7 @@ interface RLPxServiceConfiguration {
 interface DNSConfiguration {
   fun enrLink(): String
   fun pollingPeriod(): Long
+  fun dnsServer(): String?
   fun getName(): String
   fun peerRepository(): String
 }
@@ -421,14 +458,30 @@ interface PeerRepositoryConfiguration {
 }
 
 enum class SynchronizerType {
-  best, status, parent
+  best, status, parent, canonical
 }
 
 interface SynchronizerConfiguration {
   fun getName(): String
+  fun getRepository(): String
+  fun getPeerRepository(): String
+  fun getRlpxService(): String
   fun getType(): SynchronizerType
   fun getFrom(): UInt256?
   fun getTo(): UInt256?
+  fun getFromRepository(): String?
+}
+
+enum class ValidatorType {
+  chainId, transactionsHash
+}
+
+interface ValidatorConfiguration {
+  fun getName(): String
+  fun getType(): ValidatorType
+  fun getFrom(): UInt256?
+  fun getTo(): UInt256?
+  fun getChainId(): Int?
 }
 
 internal class PeerRepositoryConfigurationImpl(private val repoName: String, private val type: String) :
@@ -474,7 +527,9 @@ internal data class GenesisFileConfigurationImpl(private val name: String, priva
   override fun genesisFile(): GenesisFile =
     GenesisFile.read(
       if (genesisFilePath.scheme == "classpath") {
-        GenesisFileConfigurationImpl::class.java.getResource(genesisFilePath.path).readBytes()
+        val resource = GenesisFileConfigurationImpl::class.java.getResource(genesisFilePath.path)
+          ?: throw IllegalArgumentException("No such classpath resource: ${genesisFilePath.path}")
+        resource.readBytes()
       } else {
         Files.readAllBytes(Path.of(genesisFilePath))
       }
@@ -498,6 +553,7 @@ data class DNSConfigurationImpl(
   private val peerRepository: String,
   private val enrLink: String,
   private val pollingPeriod: Long,
+  private val dnsServer: String?
 ) :
   DNSConfiguration {
   override fun getName() = name
@@ -507,6 +563,8 @@ data class DNSConfigurationImpl(
   override fun enrLink() = enrLink
 
   override fun pollingPeriod(): Long = pollingPeriod
+
+  override fun dnsServer(): String? = dnsServer
 }
 
 data class DiscoveryConfigurationImpl(
@@ -546,11 +604,33 @@ data class ProxyConfigurationImpl(
 data class SynchronizerConfigurationImpl(
   private val name: String,
   private val type: SynchronizerType,
+  private val repository: String,
+  private val peerRepository: String,
+  private val rlpxService: String,
   private val from: UInt256?,
   private val to: UInt256?,
+  private val fromRepository: String?,
 ) : SynchronizerConfiguration {
+  override fun getName() = name
+  override fun getRepository() = repository
+  override fun getPeerRepository() = peerRepository
+  override fun getRlpxService() = rlpxService
+  override fun getType() = type
+  override fun getFrom(): UInt256? = from
+  override fun getTo(): UInt256? = to
+  override fun getFromRepository() = fromRepository
+}
+
+data class ValidatorConfigurationImpl(
+  private val name: String,
+  private val type: ValidatorType,
+  private val from: UInt256?,
+  private val to: UInt256?,
+  private val chainId: Int?,
+) : ValidatorConfiguration {
   override fun getName() = name
   override fun getType() = type
   override fun getFrom(): UInt256? = from
   override fun getTo(): UInt256? = to
+  override fun getChainId() = chainId
 }
