@@ -16,7 +16,4 @@
  * These classes are included in the standard Tuweni distribution, or separately when using the gradle dependency
  * 'org.apache.tuweni:tuweni-eth' (tuweni-eth.jar).
  */
-@ParametersAreNonnullByDefault
 package org.apache.tuweni.eth;
-
-import javax.annotation.ParametersAreNonnullByDefault;
