@@ -19,11 +19,18 @@ package org.apache.tuweni.evm
 import org.apache.tuweni.bytes.Bytes
 import org.apache.tuweni.bytes.Bytes32
 import org.apache.tuweni.eth.Address
+import org.apache.tuweni.eth.Log
+import org.apache.tuweni.eth.precompiles.PrecompileContract
 import org.apache.tuweni.eth.repository.BlockchainRepository
+import org.apache.tuweni.eth.repository.StateRepository
 import org.apache.tuweni.evm.impl.GasManager
+import org.apache.tuweni.evm.impl.Memory
+import org.apache.tuweni.evm.impl.Stack
+import org.apache.tuweni.rlp.RLP
 import org.apache.tuweni.units.bigints.UInt256
 import org.apache.tuweni.units.ethereum.Gas
 import org.apache.tuweni.units.ethereum.Wei
+import org.slf4j.LoggerFactory
 
 /**
  * Types of EVM calls
@@ -33,7 +40,8 @@ enum class CallKind(val number: Int) {
   DELEGATECALL(1),
   CALLCODE(2),
   CREATE(3),
-  CREATE2(4)
+  CREATE2(4),
+  STATICCALL(5)
 }
 
 /**
@@ -59,7 +67,8 @@ enum class EVMExecutionStatusCode(val number: Int) {
   WASM_TRAP(16),
   INTERNAL_ERROR(-1),
   REJECTED(-2),
-  OUT_OF_MEMORY(-3);
+  OUT_OF_MEMORY(-3),
+  HALTED(-4);
 }
 
 /**
@@ -88,32 +97,73 @@ enum class HardFork(val number: Int) {
 
 val latestHardFork = HardFork.BERLIN
 
+/**
+ * State of the EVM
+ *
+ * @param gasManager the gas manager
+ * @param logs the logs of the EVM
+ * @param stack the stack of the execution
+ * @param memory the current memory allocation
+ * @param output the output of the execution
+ */
+data class EVMState(
+  val gasManager: GasManager,
+  val logs: List<Log>,
+  val stack: Stack,
+  val memory: Memory,
+  val output: Bytes? = null,
+) {
+
+  /**
+   * Dumps the EVM execution state into a byte array
+   */
+  fun toBytes(): Bytes = RLP.encodeList {
+    it.writeString("gas")
+    it.writeValue(gasManager.gas.toBytes())
+    it.writeString("memory")
+    it.writeValue(memory.memoryData ?: Bytes.EMPTY)
+    it.writeString("stack")
+    for (i in 0 until stack.size()) {
+      it.writeValue(stack.get(i) ?: Bytes.EMPTY)
+    }
+    it.writeString("output")
+    it.writeValue(output ?: Bytes.EMPTY)
+    it.writeString("logs")
+    for (log in logs) {
+      it.writeValue(log.toBytes())
+    }
+  }
+}
+
 /**
  * Result of EVM execution
  * @param statusCode the execution result status
  * @param hostContext the context of changes
- * @param output the output of the execution
+ * @param changes the set of changes
+ * @param state the EVM state
  */
 data class EVMResult(
   val statusCode: EVMExecutionStatusCode,
-  val gasManager: GasManager,
   val hostContext: HostContext,
-  val output: Bytes? = null,
+  val changes: ExecutionChanges,
+  val state: EVMState,
 )
 
 /**
  * Message sent to the EVM for execution
  */
 data class EVMMessage(
-  val kind: Int,
+  val kind: CallKind,
   val flags: Int,
   val depth: Int = 0,
   val gas: Gas,
+  val contract: Address,
   val destination: Address,
   val sender: Address,
+  val origin: Address,
   val inputData: Bytes,
   val value: Bytes,
-  val createSalt: Bytes32 = Bytes32.ZERO
+  val createSalt: Bytes32 = Bytes32.ZERO,
 )
 
 /**
@@ -124,11 +174,19 @@ data class EVMMessage(
  * @param options the options to set on the EVM, specific to the library
  */
 class EthereumVirtualMachine(
+  private val transientRepository: StateRepository,
   private val repository: BlockchainRepository,
+  val precompiles: Map<Address, PrecompileContract>,
   private val evmVmFactory: () -> EvmVm,
-  private val options: Map<String, String> = mapOf()
+  private val options: Map<String, String> = mapOf(),
 ) {
 
+  companion object {
+    val logger = LoggerFactory.getLogger(EthereumVirtualMachine::class.java)
+
+    private val MAX_NONCE = UInt256.fromHexString("0xffffffffffffffff")
+  }
+
   private var vm: EvmVm? = null
 
   private fun vm() = vm!!
@@ -170,6 +228,7 @@ class EthereumVirtualMachine(
    * @param currentTimestamp current block timestamp
    * @param currentGasLimit current gas limit
    * @param currentDifficulty block current total difficulty
+   * @param chainId the ID of the current chain
    * @param callKind the type of call
    * @param revision the hard fork revision in which to execute
    * @return the result of the execution
@@ -183,18 +242,19 @@ class EthereumVirtualMachine(
     gas: Gas,
     gasPrice: Wei,
     currentCoinbase: Address,
-    currentNumber: Long,
-    currentTimestamp: Long,
+    currentNumber: UInt256,
+    currentTimestamp: UInt256,
     currentGasLimit: Long,
     currentDifficulty: UInt256,
-    callKind: CallKind = CallKind.CALL,
+    chainId: UInt256,
+    callKind: CallKind,
     revision: HardFork = latestHardFork,
-    depth: Int = 0
+    depth: Int = 0,
   ): EVMResult {
     val hostContext = TransactionalEVMHostContext(
       repository,
+      transientRepository,
       this,
-      depth,
       sender,
       destination,
       value,
@@ -205,11 +265,14 @@ class EthereumVirtualMachine(
       currentNumber,
       currentTimestamp,
       currentGasLimit,
-      currentDifficulty
+      currentDifficulty,
+      chainId,
     )
     val result =
       executeInternal(
         sender,
+        sender,
+        destination,
         destination,
         value,
         code,
@@ -225,29 +288,75 @@ class EthereumVirtualMachine(
   }
 
   internal suspend fun executeInternal(
+    origin: Address,
     sender: Address,
     destination: Address,
+    contractAddress: Address,
     value: Bytes,
     code: Bytes,
     inputData: Bytes,
     gas: Gas,
-    callKind: CallKind = CallKind.CALL,
+    callKind: CallKind,
     revision: HardFork = latestHardFork,
     depth: Int = 0,
-    hostContext: HostContext
+    hostContext: HostContext,
   ): EVMResult {
-    val msg =
-      EVMMessage(
-        callKind.number, 0, depth, gas, destination, sender, inputData,
-        value
-      )
+    var gasAvailable = gas
+    if (depth > 1024) {
+      val gasManager = GasManager(gas)
+      return EVMResult(EVMExecutionStatusCode.CALL_DEPTH_EXCEEDED, hostContext, NoOpExecutionChanges, EVMState(gasManager, listOf(), Stack(), Memory(), null))
+    }
+    val senderNonce = hostContext.getNonce(sender)
 
-    return vm().execute(
-      hostContext,
-      revision,
-      msg,
-      code
-    )
+    if (senderNonce >= MAX_NONCE) {
+      return EVMResult(EVMExecutionStatusCode.REJECTED, hostContext, NoOpExecutionChanges, EVMState(GasManager(gas), listOf(), Stack(), Memory(), null))
+    }
+
+    if (callKind == CallKind.CREATE || callKind == CallKind.CREATE2) {
+
+      val codeDepositGasFee = UInt256.valueOf(code.size() * 200L)
+      if (gas < codeDepositGasFee) {
+        return EVMResult(EVMExecutionStatusCode.REJECTED, hostContext, NoOpExecutionChanges, EVMState(GasManager(gas), listOf(), Stack(), Memory(), null))
+      }
+      gasAvailable = gasAvailable.subtract(codeDepositGasFee)
+    }
+
+    val precompile = precompiles[contractAddress]
+
+    if (!value.isZero && !options.containsKey("DISABLE_TRANSFER_VALUE")) {
+      val destinationBalance = hostContext.getBalance(destination)
+      val senderBalance = hostContext.getBalance(sender)
+      val amount = Wei.valueOf(UInt256.fromBytes(value))
+      if (senderBalance.toUInt256() < amount.toUInt256()) {
+        return EVMResult(EVMExecutionStatusCode.REJECTED, hostContext, NoOpExecutionChanges, EVMState(GasManager(gas), listOf(), Stack(), Memory(), null))
+      }
+      hostContext.setBalance(sender, senderBalance.subtract(amount))
+      hostContext.setBalance(destination, destinationBalance.add(amount))
+    }
+
+    if (precompile != null) {
+      if (callKind == CallKind.CREATE || callKind == CallKind.CREATE2) {
+        return EVMResult(EVMExecutionStatusCode.REJECTED, hostContext, NoOpExecutionChanges, EVMState(GasManager(gas), listOf(), Stack(), Memory(), null))
+      }
+      logger.trace("Executing precompile $contractAddress")
+      val result = precompile.run(inputData)
+      val gasManager = GasManager(gasAvailable)
+      gasManager.add(result.gas)
+      return EVMResult(if (result.output == null) EVMExecutionStatusCode.PRECOMPILE_FAILURE else EVMExecutionStatusCode.SUCCESS, hostContext, NoOpExecutionChanges, EVMState(gasManager, listOf(), Stack(), Memory(), result.output))
+    } else {
+      val msg =
+        EVMMessage(
+          callKind, 0, depth, gas, contractAddress, destination, sender, origin, inputData,
+          value
+        )
+
+      return vm().execute(
+        hostContext,
+        revision,
+        msg,
+        code
+      )
+    }
   }
 
   /**
@@ -284,7 +393,7 @@ interface HostContext {
    * @param key The index of the account's storage entry.
    * @return The storage value at the given storage key or null bytes if the account does not exist.
    */
-  suspend fun getRepositoryStorage(address: Address, keyBytes: Bytes): Bytes32
+  suspend fun getRepositoryStorage(address: Address, key: Bytes32): Bytes?
 
   /**
    * Get storage function.
@@ -296,7 +405,7 @@ interface HostContext {
    * @param key The index of the account's storage entry.
    * @return The storage value at the given storage key or null bytes if the account does not exist.
    */
-  suspend fun getStorage(address: Address, key: Bytes32): Bytes32
+  suspend fun getStorage(address: Address, key: Bytes): Bytes32?
 
   /**
    * Set storage function.
@@ -312,7 +421,7 @@ interface HostContext {
    * @param value The value to be stored.
    * @return The effect on the storage item.
    */
-  suspend fun setStorage(address: Address, key: Bytes, value: Bytes32): Int
+  suspend fun setStorage(address: Address, key: Bytes, value: Bytes): Int
 
   /**
    * Get balance function.
@@ -350,6 +459,18 @@ interface HostContext {
    */
   suspend fun getCodeHash(address: Address): Bytes32
 
+  /**
+   * Get account nonce
+   *
+   *
+   * This function is used by a VM to get the nonce of the account at
+   * the given address.
+   *
+   * @param address The address of the account.
+   * @return The nonce of the accountt.
+   */
+  suspend fun getNonce(address: Address): UInt256
+
   /**
    * Copy code function.
    *
@@ -384,6 +505,14 @@ interface HostContext {
    */
   suspend fun call(evmMessage: EVMMessage): EVMResult
 
+  /**
+   * This function supports EVM create calls.
+   *
+   * @param msg The create parameters.
+   * @return The result of the call.
+   */
+  suspend fun create(evmMessage: EVMMessage, code: Bytes): EVMResult
+
   /**
    * Get transaction context function.
    *
@@ -405,7 +534,7 @@ interface HostContext {
    * @param number The block hash.
    * @return The block hash or zeroed bytes if the information about the block is not available.
    */
-  fun getBlockHash(number: Long): Bytes32
+  fun getBlockHash(number: UInt256): Bytes32
 
   /**
    * Log function.
@@ -422,12 +551,12 @@ interface HostContext {
   fun emitLog(address: Address, data: Bytes, topics: List<Bytes32>)
 
   /**
-   * Returns true if the account was never used.
+   * Returns true if the account was warmed up.
    */
   fun warmUpAccount(address: Address): Boolean
 
   /**
-   * Returns true if the storage slot was never used.
+   * Returns true if the storage slot was warmed up.
    */
   fun warmUpStorage(address: Address, key: UInt256): Boolean
 
@@ -441,12 +570,16 @@ interface HostContext {
    */
   fun timestamp(): UInt256
   fun getGasLimit(): Long
-  fun getBlockNumber(): Long
+  fun getBlockNumber(): UInt256
   fun getBlockHash(): Bytes32
   fun getCoinbase(): Address
   fun getDifficulty(): UInt256
-  fun increaseBalance(address: Address, amount: Wei)
+  fun getChaindId(): UInt256
   suspend fun setBalance(address: Address, balance: Wei)
+  fun addRefund(address: Address, refund: Wei)
+  fun addRefund(address: Address, refund: Long)
+  suspend fun isEmptyAccount(address: Address): Boolean
+  suspend fun incrementNonce(address: Address): UInt256
 }
 
 interface EvmVm {
@@ -482,6 +615,9 @@ val opcodes = mapOf<Byte, String>(
   Pair(0x18, "xor"),
   Pair(0x19, "not"),
   Pair(0x1a, "byte"),
+  Pair(0x1b, "shl"),
+  Pair(0x1c, "shr"),
+  Pair(0x1d, "sar"),
   Pair(0x20, "sha3"),
   Pair(0x30, "address"),
   Pair(0x31, "balance"),
@@ -505,6 +641,8 @@ val opcodes = mapOf<Byte, String>(
   Pair(0x43, "number"),
   Pair(0x44, "difficulty"),
   Pair(0x45, "gaslimit"),
+  Pair(0x46, "chainid"),
+  Pair(0x47, "selfbalance"),
   Pair(0x50, "pop"),
   Pair(0x51, "mload"),
   Pair(0x52, "mstore"),
@@ -549,6 +687,22 @@ val opcodes = mapOf<Byte, String>(
   Pair(0x7d, "push30"),
   Pair(0x7e, "push31"),
   Pair(0x7f, "push32"),
+  Pair(0x80.toByte(), "dup1"),
+  Pair(0x81.toByte(), "dup2"),
+  Pair(0x82.toByte(), "dup3"),
+  Pair(0x83.toByte(), "dup4"),
+  Pair(0x84.toByte(), "dup5"),
+  Pair(0x85.toByte(), "dup6"),
+  Pair(0x86.toByte(), "dup7"),
+  Pair(0x87.toByte(), "dup8"),
+  Pair(0x88.toByte(), "dup9"),
+  Pair(0x89.toByte(), "dup10"),
+  Pair(0x8a.toByte(), "dup11"),
+  Pair(0x8b.toByte(), "dup12"),
+  Pair(0x8c.toByte(), "dup13"),
+  Pair(0x8d.toByte(), "dup14"),
+  Pair(0x8e.toByte(), "dup15"),
+  Pair(0x8f.toByte(), "dup16"),
   Pair(0x90.toByte(), "swap1"),
   Pair(0x91.toByte(), "swap2"),
   Pair(0x92.toByte(), "swap3"),
@@ -565,12 +719,46 @@ val opcodes = mapOf<Byte, String>(
   Pair(0x9d.toByte(), "swap14"),
   Pair(0x9e.toByte(), "swap15"),
   Pair(0x9f.toByte(), "swap16"),
-  Pair(0xf3.toByte(), "return"),
   Pair(0xa0.toByte(), "log0"),
   Pair(0xa1.toByte(), "log1"),
   Pair(0xa2.toByte(), "log2"),
   Pair(0xa3.toByte(), "log3"),
   Pair(0xa4.toByte(), "log4"),
+  Pair(0xf0.toByte(), "create"),
+  Pair(0xf1.toByte(), "call"),
+  Pair(0xf2.toByte(), "callcode"),
+  Pair(0xf3.toByte(), "return"),
+  Pair(0xf4.toByte(), "delegatecall"),
+  Pair(0xf5.toByte(), "create2"),
+  Pair(0xf6.toByte(), "unused"),
+  Pair(0xf7.toByte(), "unused"),
+  Pair(0xf8.toByte(), "unused"),
+  Pair(0xf9.toByte(), "unused"),
+  Pair(0xfa.toByte(), "staticcall"),
+  Pair(0xfb.toByte(), "unused"),
+  Pair(0xfd.toByte(), "revert"),
   Pair(0xfe.toByte(), "invalid"),
   Pair(0xff.toByte(), "selfdestruct"),
 )
+
+/**
+ * EVM transaction changes
+ */
+interface ExecutionChanges {
+
+  /**
+   * Logs emitted during execution
+   */
+  fun getLogs(): List<Log>
+
+  /**
+   * Lists of accounts to destroy
+   */
+  fun accountsToDestroy(): List<Address>
+}
+
+object NoOpExecutionChanges : ExecutionChanges {
+  override fun getLogs(): List<Log> = listOf()
+
+  override fun accountsToDestroy(): List<Address> = listOf()
+}
